#c #linux #windows

# Linux

Рассмотрим некоторые команды и библиотеки для работы с таким важным системным компонентом.

## Библиотеки

```c
#include <unistd.h> // fork
#include <stdio.h> 
#include <stdlib.h>  //для функции exit()
#include <sys/wait.h> // для wait
```

## Функции

### fork()

Это системный вызов который позволяет одному процессу, **родителю**, создать новый - дочерний процесс.

От английского слова "вилка" или "разветвление", такой термин возник т.к. родитель, как бы делится на две копии самого себя.

#### Как устроен

Эти процессы являются (почти) идентичными:

* Потомок получает копии:
	* Стека
	* Кучи
	* Текста

**Родитель возвращает идентификатор потомка при успешном выполнении**.

**Оба процесса способным выполнять один и тот же программный код (после вызова fork() )**.

Каждый процесс способен изменить свои переменные, по усмотрению.

### exit()

Завершает процесс, делая его ресурсы (память, дескрипторы), доступным для последующего перераспределения ядром.

```c
exit(status);
```

Аргумент "status" - **целое число, которое определяет код завершения процесса.**

### wait()

Через это команду родитель может извлекать код, с которым завершилась программа потомок.

```c
wait(&status)
```

Это функция имеет два значения:

* Функция приостанавливает работу, того процесса из которого она вызвана, пока не будет завершён процесс его потомка.
* Код завершения возвращается через аргумент функции wait()

### waitpid()

Снимает ряд ограничений присущий [[Процессы#wait()|wait()]]

* Если род. процесс создал несколько потомков, wait() не позволяет ожидать завершения конкретного из них; Мы можем отслеживать завершение работы только следующего дочернего процесса.
* Если ни один из потомков еще не был завершен, вызов wait() всегда блокируется. Иногда имеет смысл организовывать неблокирующее ожидание, чтобы если все потомки все ещё работают немедленно узнать об этом.
* Можно отслеживать только потомки, которые завершились (через wait). Если потомок был остановлен (сигналом) или возобновил свою работу, мы об это не узнаем.

**Сигнатура:**

```c
pid_t waitpid(pid_t pid, int *status, int optons)
```

#### Как работает

##### PID

Аргумент **pid** позволяет выбрать потомка\ов, которые будут отслеживаться

* **Если pid > 0**, ожидаем потомка, чей идентификатор равен pid.
* **Если pid == 0**, ожидаем потомка, входящего в ту же группу процессов, что и родитель..
* **Если pid < -1**, ожидаем любого потомка, идентификатор группы процессов которого равен значению "**pid по модулю"**." 
* **Если pid == -1**, ожидаем любого потомка. Вызов wait(&status) эквивалентен waitpid(01,  &status, 0).

##### OPTIONS

Аргумент **options** - битовая маска, которая может быть как пустой, так и содержать следующие флаги.

* WUNTRACED - позволяет не только узнать о завершенных дочерних процессах, но и о потомка, остановленных с помощью сигнала.
* WCONTINUED - позволяет узнать о потомках, работа которых была возобновлена после остановки в результате сигнала **SIGCOUNT**
* WNOHANG - если один из потомков, указанных в pid, все ещё не изменил свое состояние, вызов немедленно возвращается, не переходя к блокированию (т.е. опрашивает потомков). В этом случае возвращаемое значение равно 0. Если у родителей не оказывается дочерних процессов, соответствующих значению pid, waitpid() возвращает ошибку ECHILD.

### Аргумент STATUS

Этот аргумент позволяет отличать следующие события:

* Потомок завершил работу с помощью вызова exit(), вернув целочисленный код выхода.
* Потомок был завершён путем доставки необработанного сигнала.
* Потомок был остановлен сигналом, а вызов waitpid() был произведен с использованием флага WCONTINUED.

Файл <sys/wait.h> определяет стандартный набор макросов, с помощью которых можно анализировать код завершения:

* WIFEXITED - возвращает true, если дочерний процесс завершился штатно. В этом случае макрос **WEXITSTATUS**  вернет завершения дочернего процесса.

* WIFSIGNALED - возвращает true, если дочерний процесс был завершен с помощью сигнала. В этом случае макрос **WTERMSIG** возвращает номер сигнала, приведшего к завершению процесса, а макрос **WCOREDUMP** вернет true, если потомок сгенерировал файл с дампом памяти.

* WIFSTOPPED - возвращает true, если дочерний процесс был остановлен по сигналу. В этом случае макрос WSTOPSIG(status) возвращает номер сигнала, остановившего процесс.

* IFCONTINUED(status) — возвращает true, если дочерний процесс возобновил свою работу, получив сигнал SIGCONT.

### execve()

```c
execve(pathname, argv, envp);
```

Это программа загружает в память процесса новую программу по пути **pathname**, с аргументами **argv** и списком переменных среды **envp**.

Текст существующей программы сбрасывается, для новой программы заново создаются сегменты со стеком, данными и кучей. Эту операцию часто называют **выполнением новой программы**.

# WINDOWS


